@@ -1,21 +1,21 @@
-// Copyright (C) 1999,2000 Bruce Guenter <bruce@untroubled.org>
+// Copyright (C) 1999,2000,2005 Bruce Guenter <bruce@untroubled.org>
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
 //
-// This program is distributed in the hope that it will be useful,
+// This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// Lesser General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// You should have received a copy of the GNU Lesser General Public
+// License along with this library; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 #include <config.h>
-#include "ac/systime.h"
+#include "ac/time.h"
 #include "fdbuf/fdbuf.h"
 #include <stdlib.h>
 #include <string.h>
@@ -36,6 +36,7 @@ static cli_option help_option = { 'h', "help", cli_option::flag,
 
 static cli_option** options;
 static unsigned optionc;
+static const char* short_options;
 
 static void build_options()
 {
@@ -47,16 +48,23 @@ static void build_options()
   for(unsigned i = 0; i < optionc-1; i++)
     options[i] = &cli_options[i];
   options[optionc-1] = &help_option;
+
+  char* so;
+  short_options = so = new char[optionc+1];
+  for (unsigned i = 0; i < optionc; i++)
+    if (options[i]->ch != 0)
+      *so++ = options[i]->ch;
+  *so = 0;
 }
 
-static inline unsigned max(unsigned a, unsigned b)
+static inline int max(int a, int b)
 {
   return (a>b) ? a : b;
 }
 
-static const char* fill(unsigned i)
+static const char* fill(int i)
 {
-  static unsigned lastlen = 0;
+  static int lastlen = 0;
   static char* buf = 0;
   if(i > lastlen) {
     delete[] buf;
@@ -73,39 +81,43 @@ static void show_usage()
   fout << "usage: " << cli_program << " [flags] " << cli_args_usage << endl;
 }
 
-static unsigned calc_max_width()
+int calc_width(const cli_option* o)
 {
-  // maxwidth is the maximum width of the long argument
-  unsigned maxwidth = 0;
-  for(unsigned i = 0; i < optionc; i++) {
-    unsigned width = 0;
-    cli_option* o = options[i];
-    if(o->name) {
-      width += strlen(o->name);
-      switch(o->type) {
-      case cli_option::string:     width += 6; break;
-      case cli_option::integer:    width += 4; break;
-      case cli_option::uinteger:   width += 4; break;
-      case cli_option::stringlist: width += 5; break;
-      case cli_option::flag:       break;
-      case cli_option::counter:    break;
-      }
+  int width = (o->ch || !cli_only_long) ? 4 : 2;
+  if (o->name) {
+    width += strlen(o->name) + 2 + !cli_only_long;
+    switch (o->type) {
+    case cli_option::string:     width += 6; break;
+    case cli_option::integer:    width += 4; break;
+    case cli_option::uinteger:   width += 4; break;
+    case cli_option::stringlist: width += 5; break;
+    case cli_option::flag:       break;
+    case cli_option::counter:    break;
     }
-    if(width > maxwidth)
-      maxwidth = width;
   }
+  return width;
+}
+
+static int calc_max_width()
+{
+  // maxwidth is the maximum width of the option text prefix
+  int maxwidth = 0;
+  for(unsigned i = 0; i < optionc; i++)
+    maxwidth = max(maxwidth, calc_width(options[i]));
   return maxwidth;
 }
 
-static void show_option(cli_option* o, unsigned maxwidth)
+static void show_option(const cli_option* o, int maxwidth)
 {
   if(o == &help_option)
     fout << '\n';
+  fout << "  ";
   if(o->ch)
-    fout << "  -" << o->ch;
-  else
-    fout << "    ";
-  fout << (o->ch && o->name ? ", " : "  ");
+    fout << '-' << o->ch;
+  else if (!cli_only_long)
+    fout << "  ";
+  if (o->ch || !cli_only_long)
+    fout << (o->ch && o->name ? ", " : "  ");
   if(o->name) {
     const char* extra = "";
     switch(o->type) {
@@ -116,21 +128,22 @@ static void show_option(cli_option* o, unsigned maxwidth)
     case cli_option::flag:       break;
     case cli_option::counter:    break;
     }
-    fout << "--" << o->name << extra
-	 << fill(maxwidth - strlen(o->name) - strlen(extra) + 2);
+    fout << (cli_only_long ? "-" : "--") << o->name << extra
+	 << fill(maxwidth - strlen(o->name) - strlen(extra) - !cli_only_long
+		 - (o->ch || !cli_only_long ? 4 : 0));
   }
   else
-    fout << fill(maxwidth+4);
+    fout << fill(maxwidth-3);
   fout << o->helpstr << '\n';
   if(o->defaultstr)
-    fout << fill(maxwidth+10) << "(Defaults to " << o->defaultstr << ")\n";
+    fout << fill(maxwidth+3) << "(Defaults to " << o->defaultstr << ")\n";
 }
 
 static void show_help()
 {
   if(cli_help_prefix)
     fout << cli_help_prefix;
-  unsigned maxwidth = calc_max_width();
+  int maxwidth = calc_max_width();
   for(unsigned i = 0; i < optionc; i++)
     show_option(options[i], maxwidth);
   if(cli_help_suffix)
@@ -230,49 +243,67 @@ static int parse_short(int argc, char* argv[])
   return 0;
 }
 
-int cli_option::parse_long_eq(const char* arg)
+static void option_error(const cli_option* o, int as_short, const char* text)
+{
+  ferr << argv0 << ": option ";
+  if (as_short)
+    ferr << '-' << o->ch;
+  else
+    ferr << (cli_only_long ? "-" : "--") << o->name;
+  ferr << text << endl;
+}
+
+int cli_option::parse_long_eq(const char* arg, int as_short)
 {
   if(type == flag || type == counter) {
-    ferr << argv0 << ": option --" << name
-	 << " does not take a value." << endl;
+    option_error(this, as_short, " does not take a value.");
     return -1;
   }
   else
     return set(arg)-1;
 }
 
-int cli_option::parse_long_noeq(const char* arg)
+int cli_option::parse_long_noeq(const char* arg, int as_short)
 {
   if(type == flag || type == counter)
     return set(0);
   else if(arg)
     return set(arg);
   else {
-    ferr << argv0 << ": option --" << name
-	 << " requires a value." << endl;
+    option_error(this, as_short, " requires a value.");
     return -1;
   }
 }
 
 static int parse_long(int, char* argv[])
 {
-  const char* arg = argv[0]+2;
+  const char* arg = argv[0]+1;
+  // Handle both short and long args
+  if (arg[0] == '-')
+    ++arg;
   for(unsigned j = 0; j < optionc; j++) {
     cli_option* o = options[j];
     if(o->name) {
       size_t len = strlen(o->name);
       if(!memcmp(arg, o->name, len)) {
 	if(arg[len] == '\0')
-	  return o->parse_long_noeq(argv[1]);
+	  return o->parse_long_noeq(argv[1], false);
 	else if(arg[len] == '=')
-	  return o->parse_long_eq(arg+len+1);
+	  return o->parse_long_eq(arg+len+1, false);
       }
     }
   }
-  ferr << argv0 << ": unknown option string: '--" << arg << "'" << endl;
+  ferr << argv0 << ": unknown option string: '" << argv[0] << "'" << endl;
   return -1;
 }
 
+static int parse_either(int argc, char* argv[])
+{
+  return (strchr(short_options, argv[0][1]) != 0)
+    ? parse_short(argc, argv)
+    : parse_long(argc, argv);
+}
+
 static int parse_args(int argc, char* argv[])
 {
   build_options();
@@ -288,9 +319,9 @@ static int parse_args(int argc, char* argv[])
       i++;
       break;
     }
-    int j = (arg[1] != '-') ?
-      parse_short(argc-i, argv+i) :
-      parse_long(argc-i, argv+i);
+    int j = (arg[1] == '-') ? parse_long(argc-i, argv+i)
+      : cli_only_long ? parse_either(argc-i, argv+i)
+      : parse_short(argc-i, argv+i);
     if(j < 0)
       usage(1);
     else
