@@ -1,5 +1,5 @@
 // nullmailer -- a simple relay-only MTA
-// Copyright (C) 2007  Bruce Guenter <bruce@untroubled.org>
+// Copyright (C) 2012  Bruce Guenter <bruce@untroubled.org>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -84,19 +84,21 @@ int selfpipe::caught()
 
 int selfpipe::waitsig(int timeout)
 {
-  fd_set fdset;
-  FD_ZERO(&fdset);
-  FD_SET(fds[0], &fdset);
-  struct timeval tv;
-  tv.tv_sec = timeout;
-  tv.tv_usec = 0;
-  int s;
-  while ((s = select(fds[0] + 1, &fdset, 0, 0,
+  if (timeout > 0) {
+    fd_set fdset;
+    FD_ZERO(&fdset);
+    FD_SET(fds[0], &fdset);
+    struct timeval tv;
+    tv.tv_sec = timeout;
+    tv.tv_usec = 0;
+    int s;
+    while ((s = select(fds[0] + 1, &fdset, 0, 0,
 		     (timeout <= 0) ? 0 : &tv)) == -1) {
-    if (errno != EINTR)
-      return -1;
+      if (errno != EINTR)
+        return -1;
+    }
+    if (s != 1)
+      return 0;
   }
-  if (s == 1)
-    return caught();
-  return 0;
+  return caught();
 }
