@@ -1,5 +1,5 @@
 // nullmailer -- a simple relay-only MTA
-// Copyright (C) 2007  Bruce Guenter <bruce@untroubled.org>
+// Copyright (C) 2012  Bruce Guenter <bruce@untroubled.org>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -29,7 +29,10 @@
 
 const char* user = 0;
 const char* pass = 0;
-int auth_method = AUTH_PLAIN;
+int port = 0;
+int auth_method = AUTH_DETECT;
+int use_ssl = 0;
+int use_starttls = 0;
 const char* cli_help_suffix = "";
 const char* cli_args_usage = "remote-address < mail-file";
 const int cli_args_min = 1;
@@ -42,7 +45,23 @@ cli_option cli_options[] = {
   { 0, "pass", cli_option::string, 0, &pass,
     "Set the password for authentication", 0 },
   { 0, "auth-login", cli_option::flag, AUTH_LOGIN, &auth_method,
-    "Use AUTH LOGIN instead of AUTH PLAIN in SMTP", 0 },
+    "Use AUTH LOGIN instead of auto-detecting in SMTP", 0 },
+#ifdef HAVE_TLS
+  { 0, "ssl", cli_option::flag, 1, &use_ssl,
+    "Connect using SSL (on an alternate port by default)", 0 },
+  { 0, "starttls", cli_option::flag, 1, &use_starttls,
+    "Use STARTTLS command", 0 },
+  { 0, "x509certfile", cli_option::string, 0, &tls_x509certfile,
+    "Client certificate file", 0 },
+  { 0, "x509cafile", cli_option::string, 0, &tls_x509cafile,
+    "Certificate authority trust file", DEFAULT_CA_FILE },
+  { 0, "x509crlfile", cli_option::string, 0, &tls_x509crlfile,
+    "Certificate revocation list file", 0 },
+  { 0, "x509fmtder", cli_option::flag, true, &tls_x509derfmt,
+    "X.509 files are in DER format", "PEM format" },
+  { 0, "insecure", cli_option::flag, true, &tls_insecure,
+    "Don't abort if server certificate fails validation", 0 },
+#endif
   {0, 0, cli_option::flag, 0, 0, 0, 0}
 };
 
@@ -58,15 +77,38 @@ void protocol_succ(const char* msg)
   exit(0);
 }
 
+static void plain_send(fdibuf& in, int fd)
+{
+  fdibuf netin(fd);
+  fdobuf netout(fd);
+  if (!netin || !netout)
+    protocol_fail(ERR_MSG_TEMPFAIL, "Error allocating I/O buffers");
+  if (use_starttls) {
+    protocol_starttls(netin, netout);
+    tls_send(in, fd);
+  }
+  else
+    protocol_send(in, netin, netout);
+}
+
 int cli_main(int, char* argv[])
 {
   const char* remote = argv[0];
+  if (port == 0)
+    port = use_ssl ? default_ssl_port : default_port;
+  if (port < 0)
+    protocol_fail(ERR_USAGE, "Invalid value for --port");
+  if (use_ssl || use_starttls)
+    tls_init(remote);
   fdibuf in(0, true);
-  protocol_prep(&in);
+  protocol_prep(in);
   int fd = tcpconnect(remote, port);
   if(fd < 0)
     protocol_fail(-fd, "Connect failed");
-  protocol_send(&in, fd);
+  if (use_ssl)
+    tls_send(in, fd);
+  else
+    plain_send(in, fd);
   return 0;
 }
 
